@@ -21,12 +21,17 @@
  */
 
 #include <config.h>
-#include <gnome.h>
+
 #include <string.h>
-#include <games-conf.h>
-#include <games-frame.h>
-#include <games-files.h>
-#include <games-sound.h>
+
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
+
+#include <libgames-support/games-conf.h>
+#include <libgames-support/games-frame.h>
+#include <libgames-support/games-files.h>
+#include <libgames-support/games-sound.h>
+#include <libgames-support/games-runtime.h>
 
 #include "properties.h"
 #include "gnothello.h"
@@ -56,7 +61,6 @@ extern gchar *tile_set_tmp;
 extern gint8 pixmaps[8][8];
 extern gint animate;
 extern gint animate_stagger;
-extern gint flip_pixmaps_id;
 extern gint flip_final;
 extern gint grid;
 gint sound;
@@ -122,15 +126,14 @@ load_properties (void)
 
   switch (animate) {
   case 0:
-    flip_pixmaps_id = g_timeout_add (100, flip_pixmaps, NULL);
+    set_animation_speed (100);
     break;
   case 1:
-    flip_pixmaps_id =
-      g_timeout_add (PIXMAP_FLIP_DELAY * 8, flip_pixmaps, NULL);
+    set_animation_speed (PIXMAP_FLIP_DELAY * 8);
     break;
   default:
   case 2:
-    flip_pixmaps_id = g_timeout_add (PIXMAP_FLIP_DELAY, flip_pixmaps, NULL);
+    set_animation_speed (PIXMAP_FLIP_DELAY);
     break;
   }
 }
@@ -293,21 +296,16 @@ apply_changes (void)
 
   animate = t_animate;
 
-  if (flip_pixmaps_id) {
-    g_source_remove (flip_pixmaps_id);
-    flip_pixmaps_id = 0;
-  }
-
   switch (animate) {
   case 0:
-    flip_pixmaps_id = g_timeout_add (100, flip_pixmaps, NULL);
+    set_animation_speed (100);
     break;
   case 1:
-    flip_pixmaps_id = g_timeout_add (PIXMAP_FLIP_DELAY * 8,
-				     flip_pixmaps, NULL);
+    set_animation_speed (PIXMAP_FLIP_DELAY * 8);
     break;
+  default:
   case 2:
-    flip_pixmaps_id = g_timeout_add (PIXMAP_FLIP_DELAY, flip_pixmaps, NULL);
+    set_animation_speed (PIXMAP_FLIP_DELAY);
     break;
   }
 
@@ -361,23 +359,20 @@ free_str (GtkWidget * widget, void *data)
 static GtkWidget *
 fill_menu (void)
 {
-  gchar *dname = NULL;
+  const char *dir;
 
   /* FIXME: we need to check that both dname is valid and that
    * games_file_list_new_images returns something. */
 
-  dname = gnome_program_locate_file (NULL,
-				     GNOME_FILE_DOMAIN_APP_PIXMAP,
-				     "iagno", FALSE, NULL);
+  dir = games_runtime_get_directory (GAMES_RUNTIME_GAME_PIXMAP_DIRECTORY);
 
   if (theme_file_list)
     g_object_unref (theme_file_list);
 
-  theme_file_list = games_file_list_new_images (dname, NULL);
-  g_free (dname);
+  theme_file_list = games_file_list_new_images (dir, NULL);
 
   games_file_list_transform_basename (theme_file_list);
-
+ 
   return games_file_list_create_widget (theme_file_list, tile_set,
 					GAMES_FILE_LIST_REMOVE_EXTENSION |
 					GAMES_FILE_LIST_REPLACE_UNDERSCORES);
